using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.OleDb;
using System.IO;
using System.Text;
using System.Diagnostics;

using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace GGC_caGroup
{
	/// <summary>
	/// GroupingEngineFactory provides a trimmed down GridGroup
	/// which eliminates overhead of not needed preview rows, header and footer cells.
	/// </summary>
	public class GroupingEngineFactory : GridEngineFactoryBase
	{
		// Add this line in your forms ctor:
		// GroupingEngineFactory provides custom group that does not display caption if only one row in group
		// GridEngineFactory.Factory = new GroupingEngineFactory();

		public override GridEngine CreateEngine()
		{
			return new GroupingEngine();
		}
	}

	public class GroupingEngine : GridEngine
	{
		public override Group CreateGroup(Section parent)
		{
			return new GroupingGroup(parent);
		}
	}

	public class GroupingGroup : Group, IGridGroupOptionsSource
	{
		public GroupingGroup(Section parent)
			: base(parent)
		{
		}

		protected override void OnInitializeSections(bool hasRecords, SortColumnDescriptorCollection fields)
		{
			// Caption
			this.Sections.Add(this.ParentTableDescriptor.CreateCaptionSection(this));

			// Details (Records or Groups)
			if (hasRecords)
				this.Sections.Add(this.ParentTableDescriptor.CreateRecordsDetails(this, fields));
			else
				this.Sections.Add(this.ParentTableDescriptor.CreateGroupsDetails(this, fields));

			// Summary
			this.Sections.Add(this.ParentTableDescriptor.CreateSummarySection(this));

		}		

		public override bool IsChildVisible(Element el)
		{
			if (el is CaptionSection)
			{
				//System.Diagnostics.Debug.WriteLine(el.ParentGroup.Category);
				if(el.ParentGroup.Category.ToString() != "None" )
					return true;
				else
					return false;
			}
			else if (el is DetailsSection)
			{
				if (el.ParentGroup.Category.ToString() == "None")
					return true;
				else
					return el.ParentGroup.IsExpanded;

			}			
			else 
				return el.ParentGroup.IsExpanded;
		}

		#region IGridGroupOptionsSource Members

		GridGroupOptionsStyleInfo groupOptions;

		public void RaiseGroupOptionsChanging(GridGroupOptionsChangedEventArgs e)
		{
		}

		public void RaiseGroupOptionsChanged(GridGroupOptionsChangedEventArgs e)
		{
			this.InvalidateCounterTopDown(true);
		}

		public bool HasGroupOptions
		{
			get
			{
				return groupOptions != null;
			}
		}

		/// <override/>
		public new GridEngine Engine
		{
			get
			{
				return (GridEngine) base.Engine;
			}
		}

		/// <summary>
		/// Lets you control the look of inner groups like whether the Caption Row is visible, or what CaptionText is.
		/// </summary>
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		[Browsable(true)]
		public GridGroupOptionsStyleInfo GroupOptions
		{
			get
			{
				if (groupOptions == null)
				{
					GridGroupOptionsType t = GridGroupOptionsType.Groups;
					if (this.IsTopLevelGroup)
						t = GridGroupOptionsType.TopLevelGroup;
					else //if (this is ChildTable)
						t = GridGroupOptionsType.ChildTable;

					groupOptions = new GridGroupOptionsStyleInfo(new GridGroupOptionsStyleInfoIdentity((IGridGroupOptionsSource) this, t));
				}
				return groupOptions;
			}
			set
			{
				GroupOptions.CopyFrom(value);
			}
		}

		public IGridGroupOptionsSource GetParentGroupOptionsSource()
		{
			if (this.CategoryColumns.Count > 0)
			{
				GridColumnDescriptor column = ((GridTableDescriptor) this.ParentTableDescriptor).Columns[this.CategoryColumns[0].Name];
				if (column != null)
					return column;
			}
			return (IGridGroupOptionsSource) this.ParentTableDescriptor;
		}

		#endregion

	}
}
